package com.fireworksdk.bridge.flutter

import android.annotation.SuppressLint
import android.app.Activity
import android.content.Context
import android.content.pm.PackageManager
import android.provider.Settings
import android.util.Base64
import com.firework.imageloading.ImageLoader
import com.firework.livestream.LivestreamPlayerInitializer
import com.firework.sdk.FireworkSdk
import com.firework.sdk.FireworkSdkConfig
import com.fireworksdk.bridge.models.FWSDKInitOptionsModel
import com.fireworksdk.bridge.models.FWSdkInitResultModel
import com.fireworksdk.bridge.models.enums.FWPlayerLaunchBehavior
import com.fireworksdk.bridge.utils.FWGlobalDataUtil
import com.fireworksdk.bridge.utils.FWImageLoaderUtil
import com.fireworksdk.bridge.utils.FWLanguageUtil
import com.fireworksdk.bridge.utils.FWLiveStreamUtil
import com.fireworksdk.bridge.utils.FWLogUtils
import java.nio.charset.StandardCharsets
import java.security.MessageDigest

object FWFlutterSDK {

  fun init(
    context: Context,
    sdkInitOptions: FWSDKInitOptionsModel? = null,
  ) {
    initSdkInternal(context, sdkInitOptions, onError = {
      FWGlobalDataUtil.sdkInitResultModel = FWSdkInitResultModel(
        success = false,
        reason = it
      )
    }, onSuccess = {
      FWGlobalDataUtil.sdkInitResultModel = FWSdkInitResultModel(
        success = true,
        reason = null
      )
    })
  }

  fun addLivestreamPlayerInitializer(initializer: LivestreamPlayerInitializer?) {
    FWLiveStreamUtil.addLivestreamPlayerInitializer(initializer)
  }

  fun setImageLoader(imageLoader: ImageLoader?) {
    FWImageLoaderUtil.setImageLoader(imageLoader)
  }

  fun updateBaseContextLocale(context: Context): Context {
    return FWLanguageUtil.getInstance(context).updateBaseContextLocale(context)
  }


  fun changeLanguage(localeString: String, activity: Activity) {
    FWLanguageUtil.getInstance(activity).changeLanguage(localeString, activity)
  }

  fun initSdkInternal(
    context: Context,
    sdkInitOptions: FWSDKInitOptionsModel?,
    onSuccess: (() -> Unit)? = null,
    onError: ((String) -> Unit)? = null
  ) {
    val clientId = getAppid(context)
    if (clientId.isNullOrBlank()) {
      val errorMsg = "The appid is invalid"
      FWLogUtils.d { errorMsg }
      onError?.invoke(errorMsg)
      return
    }
    val userId = sdkInitOptions?.userId

    val uid = if (userId.isNullOrBlank()) { getHashedAndroidId(context) } else userId
    val videoLaunchBehavior = sdkInitOptions?.videoLaunchBehavior

    val configBuilder = FireworkSdkConfig.Builder(context)
      .checksumRequired(false)
      .clientId(clientId)
      .userId(uid)
      .enableCache(true)

    val livestreamPlayerInitializerList = FWLiveStreamUtil.livestreamPlayerInitializers()
    for (livestreamPlayerInitializer in livestreamPlayerInitializerList) {
      configBuilder.addLivestreamPlayerInitializer(livestreamPlayerInitializer)
    }
    when (videoLaunchBehavior) {
      FWPlayerLaunchBehavior.Default -> {}
      FWPlayerLaunchBehavior.MuteOnFirstLaunch -> {
        configBuilder.muteOnLaunch(true)
      }
      else -> {}
    }

    val imageLoader = FWImageLoaderUtil.getImageLoader()
    if (imageLoader != null) {
      configBuilder.imageLoader(imageLoader)
    }

    val config = configBuilder.build()

    FireworkSdk.init(
      config,
      onSuccess = {
        onSuccess?.invoke()
      },
      onError = { error ->
        onError?.invoke("$error")
      },
    )
  }

  private fun getAppid(context: Context): String? {
    val ai = context.packageManager.getApplicationInfo(
      context.packageName,
      PackageManager.GET_META_DATA
    )
    val value = ai.metaData.get("Firework:Appid")
    return value?.let {
      it as String
    } ?: run {
      null
    }
  }

  @SuppressLint("HardwareIds")
  private fun getHashedAndroidId(context: Context): String {
    val androidId = Settings.Secure.getString(
      context.contentResolver,
      Settings.Secure.ANDROID_ID
    )
    return getHash(androidId)
  }

  private fun getHash(text: String): String {
    val digest: MessageDigest = MessageDigest.getInstance("SHA-256")
    val hash: ByteArray = digest.digest(text.toByteArray(StandardCharsets.UTF_8))
    return Base64.encodeToString(hash, Base64.DEFAULT)
  }

}