package com.fireworksdk.bridge.flutter.platformview

import android.view.View
import androidx.fragment.app.FragmentActivity
import com.firework.error.storyblock.StoryBlockError
import com.firework.storyblock.FeedLoadState
import com.fireworksdk.bridge.components.storyblock.StoryBlockFragment
import com.fireworksdk.bridge.components.storyblock.StoryBlockFrameLayout
import com.fireworksdk.bridge.flutter.handler.FWStoryBlockHandler
import com.fireworksdk.bridge.flutter.models.FWChannelName
import com.fireworksdk.bridge.flutter.utils.FWEventUtils
import com.fireworksdk.bridge.models.FWVideoFeedPropsModel
import com.fireworksdk.bridge.models.FWVideoFeedPropsModelDeserializer
import com.fireworksdk.bridge.utils.FWFragmentUtil
import com.fireworksdk.bridge.utils.FWLogUtils
import io.flutter.plugin.common.BinaryMessenger
import io.flutter.plugin.common.MethodChannel
import io.flutter.plugin.platform.PlatformView
import org.json.JSONObject
import java.util.Random

class FWStoryBlockPlatformView(
  private val activity: FragmentActivity,
  private val viewId: Int,
  creationParams: Map<String?, Any?>?,
  binaryMessenger: BinaryMessenger,
) : PlatformView {

  private val storyBlockFrameLayout: StoryBlockFrameLayout
  private val methodChannel = MethodChannel(
    binaryMessenger,
    FWChannelName.FireworkStoryBlock.rawValue.replace("(viewId)", "$viewId")
  )

  init {
    val creationParamsMutableMap = creationParams?.toMutableMap()
    val creationParamsJSONObject = creationParamsMutableMap?.let {
      JSONObject(it)
    }
    val videoFeedProps: FWVideoFeedPropsModel? =
      FWVideoFeedPropsModelDeserializer.deserialize(creationParamsJSONObject)

    val storyBlockFrameLayout = StoryBlockFrameLayout(activity)
    if (videoFeedProps != null) {
      storyBlockFrameLayout.videoFeedPropsModel = videoFeedProps
    }
    val randomViewId = Random().nextInt()
    storyBlockFrameLayout.id = randomViewId

    val fragment = StoryBlockFragment.newInstance(storyBlockFrameLayout.videoFeedPropsModel, true)
    storyBlockFrameLayout.storyBlockFragment = fragment
    addStoryBlockListener(fragment)

    val parentView = storyBlockFrameLayout
    if (parentView.isAttachedToWindow) {
      if (!fragment.isAdded) {
        activity.supportFragmentManager
          .beginTransaction()
          .replace(randomViewId, fragment, randomViewId.toString())
          .commit()
      }
    } else {
      parentView.addOnAttachStateChangeListener(object : View.OnAttachStateChangeListener {
        override fun onViewAttachedToWindow(v: View) {
          FWLogUtils.d { "FWStoryBlockManager createFragment doOnAttach" }

          if (!fragment.isAdded) {
            activity.supportFragmentManager
              .beginTransaction()
              .replace(randomViewId, fragment, randomViewId.toString())
              .commit()
          }
        }

        override fun onViewDetachedFromWindow(v: View) {
        }
      })
    }

    this.storyBlockFrameLayout = storyBlockFrameLayout
    methodChannel.setMethodCallHandler(FWStoryBlockHandler(activity, storyBlockFrameLayout))
  }


  override fun getView(): View {
    return storyBlockFrameLayout
  }

  override fun dispose() {
    storyBlockFrameLayout.storyBlockFragment?.removeIvsPlayerView()
    FWFragmentUtil.removeFragment(activity, storyBlockFrameLayout.storyBlockFragment)
    storyBlockFrameLayout.destroy()
  }

  private fun addStoryBlockListener(fragment: StoryBlockFragment) {
    fragment.setFeedLoadListener { feedLoadState ->
      when (feedLoadState) {
        FeedLoadState.Loading -> {
          FWLogUtils.d { "StoryBlock FeedViewState.Loading" }
        }
        FeedLoadState.FeedLoaded -> {
          FWEventUtils.sendStoryBlockLoadSuccessEvent(methodChannel)
        }
        FeedLoadState.EmptyFeed -> {
          FWEventUtils.sendStoryBlockEmptyEvent(methodChannel)
        }
        FeedLoadState.EndOfFeed -> {
          FWEventUtils.sendStoryBlockLoadSuccessEvent(methodChannel)
        }
        else -> {}
      }
    }

    fragment.setOnErrorListener { error ->
      when (error) {
        is StoryBlockError.LoadingFailed -> {
          FWEventUtils.sendStoryBlockLoadFailedEvent(
            methodChannel,
            error.message,
            error.message
          )
        }
        else -> {
          FWEventUtils.sendStoryBlockLoadFailedEvent(
            methodChannel,
            null,
            null,
          )
        }
      }
    }
  }

}