//
//  StoryBlock.swift
//  fw_flutter_sdk
//
//  Created by linjie jiang on 2022/8/10.
//

import FireworkVideo
import UIKit

@objc
public enum StoryBlockSourceType: Int {
    case discover, channel, playlist, dynamicContent, hashtagPlaylist, sku, singleContent
}

public protocol StoryBlockDelegate: AnyObject {
    func storyBlockDidLoadFeed(_ view: StoryBlock)
    func storyBlock(_ view: StoryBlock, didFailToLoadFeed error: StoryBlockError)
    func storyBlock(_ view: StoryBlock, pdpVisibilityChange visible: Bool)

    func pictureInPictureControllerDidStartPictureInPicture(
        _ pictureInPictureController: FireworkVideo.PictureInPictureController
    )

    func pictureInPictureControllerDidStopPictureInPicture(
        _ pictureInPictureController: FireworkVideo.PictureInPictureController
    )
}

@objc
// swiftlint:disable type_body_length
public class StoryBlock: UIView, StoryBlockViewControllerDelegate, PictureInPictureControllerDelegate {
    @objc public var sourceType: StoryBlockSourceType = .discover
    @objc public var channel: String = ""
    @objc public var playlist: String = ""
    @objc public var dynamicContentParameters: [String: [String]] = [:]
    @objc public var hashtagFilterExpression: String = ""
    @objc public var productIds: [String] = []
    @objc public var contentId: String = ""
    @objc public var adConfiguration: AdConfiguration?
    @objc public var storyBlockConfiguration: StoryBlockConfiguration? {
        didSet {
            guard let storyBlockVC = storyBlockVC else {
                return
            }

            if storyBlockConfiguration?.videoPlayerLogoConfiguration != oldValue?.videoPlayerLogoConfiguration {
                storyBlockVC.isPictureInPictureEnabled = false
                storyBlockVC.detachFromParent()
                self.storyBlockVC = nil
                DispatchQueue.main.async {
                    self.embed()
                }
            } else {
                updateStoryBlockVCConfiguration()
            }
        }
    }

    public weak var delegate: StoryBlockDelegate?

    private var storyBlockVC: StoryBlockViewController?

    private var source: StoryBlockContentSource {
        switch sourceType {
        case .discover:
            return .discover
        case .channel:
            return .channel(channelID: channel)
        case .playlist:
            return .channelPlaylist(channelID: channel, playlistID: playlist)
        case .dynamicContent:
            return .dynamicContent(channelID: channel, parameters: dynamicContentParameters)
        case .hashtagPlaylist:
            return .hashtagPlaylist(channelID: channel, filterExpression: hashtagFilterExpression)
        case .sku:
            return .skuPlaylist(channelID: channel, productIDs: productIds)
        case .singleContent:
            return .singleContent(videoOrLiveStreamID: contentId)
        }
    }

    private var isInPip: Bool = false

    public override init(frame: CGRect) {
        super.init(frame: frame)
        self.clipsToBounds = true
    }

    required init?(coder: NSCoder) {
        super.init(coder: coder)
        self.clipsToBounds = true
    }

    public override func layoutSubviews() {
        super.layoutSubviews()

        embed()
    }

    private func embed() {
        guard let parentVC = parentViewController else {
            return
        }
        guard self.storyBlockVC == nil else {
            return
        }

        var resultStoryBlockVC: StoryBlockViewController?
        if let fireworkVideoAdConfiguration = Convert.getFireworkVideoAdConfiguration(adConfiguration) {
            resultStoryBlockVC = StoryBlockViewController(source: source, adConfiguration: fireworkVideoAdConfiguration)
        } else {
            resultStoryBlockVC = StoryBlockViewController(source: source)
        }

        guard let storyBlockVC = resultStoryBlockVC else {
            return
        }

        self.storyBlockVC = storyBlockVC
        updateStoryBlockVCConfiguration()

        storyBlockVC.isPictureInPictureEnabled = storyBlockConfiguration?.enablePictureInPicture ?? true
        storyBlockVC.pictureInPictureDelegate = self
        storyBlockVC.delegate = self

        parentVC.attachChild(storyBlockVC, to: self)
    }

    public override func removeFromSuperview() {
        super.removeFromSuperview()

        guard let storyBlockVC = self.storyBlockVC else {
            return
        }
        if !isInPip {
            storyBlockVC.isPictureInPictureEnabled = false
        }
        storyBlockVC.detachFromParent()
        self.storyBlockVC = nil
    }

    public func play() {
        guard let storyBlockVC = self.storyBlockVC else {
            return
        }
        storyBlockVC.play()
    }

    public func pause() {
        guard let storyBlockVC = self.storyBlockVC else {
            return
        }
        storyBlockVC.pause()
    }

    public func storyBlockDidLoadFeed(_ viewController: StoryBlockViewController) {
        guard let delegate = self.delegate else {
            return
        }

        delegate.storyBlockDidLoadFeed(self)
    }

    public func storyBlock(
        _ viewController: StoryBlockViewController, didFailToLoadFeed error: StoryBlockError
    ) {
        guard let delegate = self.delegate else {
            return
        }

        delegate.storyBlock(self, didFailToLoadFeed: error)
    }

    public func notifyPdpVisibilityChange(_ visible: Bool) {
        guard let delegate = self.delegate else {
            return
        }

        delegate.storyBlock(self, pdpVisibilityChange: visible)
    }

    // MARK: PictureInPictureDelegate
    public func pictureInPictureControllerWillStartPictureInPicture(
        _ pictureInPictureController: FireworkVideo.PictureInPictureController
    ) {
    }

    public func pictureInPictureControllerDidStartPictureInPicture(
        _ pictureInPictureController: FireworkVideo.PictureInPictureController
    ) {
        guard let delegate = self.delegate else {
            return
        }

        delegate.pictureInPictureControllerDidStartPictureInPicture(pictureInPictureController)
        isInPip = true
    }

    public func pictureInPictureController(
        _ pictureInPictureController: FireworkVideo.PictureInPictureController,
        failedToStartPictureInPictureWithError error: Error?
    ) {
    }

    public func pictureInPictureControllerWillStopPictureInPicture(
        _ pictureInPictureController: FireworkVideo.PictureInPictureController
    ) {
    }

    public func pictureInPictureControllerDidStopPictureInPicture(
        _ pictureInPictureController: FireworkVideo.PictureInPictureController
    ) {
        guard let delegate = self.delegate else {
            return
        }

        delegate.pictureInPictureControllerDidStopPictureInPicture(pictureInPictureController)
        isInPip = false
    }

    public func pictureInPictureController(
        _ pictureInPictureController: FireworkVideo.PictureInPictureController,
        restoreUserInterfaceForPictureInPictureStopWithCompletionHandler completionHandler: @escaping (Bool) -> Void
    ) {
        completionHandler(true)
    }

    // swiftlint:disable:next cyclomatic_complexity function_body_length
    private func updateStoryBlockVCConfiguration() {
        guard let storyBlockVC = storyBlockVC else {
            return
        }

        var resultConfig = FireworkVideo.StoryBlockConfiguration()
        guard let config = storyBlockConfiguration else {
            storyBlockVC.viewConfiguration = resultConfig
            return
        }

        if let playerStyle = config.playerStyle {
            switch playerStyle {
            case .full:
                resultConfig.fullScreenPlayerView.playerStyle = .fullBleed
            default:
                resultConfig.fullScreenPlayerView.playerStyle = .fit
            }
        }

        if let videoCompleteAction = config.videoCompleteAction {
            switch videoCompleteAction {
            case .loop:
                resultConfig.fullScreenPlayerView.videoCompleteAction = .loop
            default:
                resultConfig.fullScreenPlayerView.videoCompleteAction = .advanceToNext
            }
        }

        if let showShareButton = config.showShareButton {
            resultConfig.shareButton.isHidden = !showShareButton
            resultConfig.fullScreenPlayerView.shareButton.isHidden = !showShareButton
        }

        if let ctaButtonStyle = config.ctaButtonStyle {
            if let backgroundColor = ctaButtonStyle.backgroundColor, let color = backgroundColor.uicolor() {
                resultConfig.fullScreenPlayerView.ctaButton.contentConfiguration.backgroundColor = color
            }
            if let textcolor = ctaButtonStyle.textColor, let color = textcolor.uicolor() {
                resultConfig.fullScreenPlayerView.ctaButton.contentConfiguration.textColor = color
            }
            if let fontSize = ctaButtonStyle.fontSize {
                let iOSFontInfo = ctaButtonStyle.iOSFontInfo ?? FontInfo()
                resultConfig.fullScreenPlayerView.ctaButton.contentConfiguration.font = iOSFontInfo.getFont(fontSize)
            }
        }

//        if let showPlaybackButton = config.showPlaybackButton {
//            resultConfig.fullScreenPlayerView.playbackButton.isHidden = !showPlaybackButton
//        }
//
//        if let showMuteButton = config.showMuteButton {
//            resultConfig.fullScreenPlayerView.muteButton.isHidden = !showMuteButton
//        }

        if let showBranding = config.showBranding {
            resultConfig.videoDetail.fireworkAttribution.isHidden = !showBranding
            resultConfig.fullScreenPlayerView.videoDetail.fireworkAttribution.isHidden = !showBranding
        }

        if let showVideoDetailTitle = config.showVideoDetailTitle {
            resultConfig.videoDetail.titleConfiguration.isHidden = !showVideoDetailTitle
            resultConfig.fullScreenPlayerView.videoDetail.titleConfiguration.isHidden = !showVideoDetailTitle
        }

        if let shareBaseURL = config.shareBaseURL {
            resultConfig.shareButton.behavior.baseURL = URL(string: shareBaseURL)
            resultConfig.fullScreenPlayerView.shareButton.behavior.baseURL = URL(string: shareBaseURL)
        } else if let shareBaseUrl = FireworkSDKPlugin.shareBaseURL {
            resultConfig.shareButton.behavior.baseURL = URL(string: shareBaseUrl)
            resultConfig.fullScreenPlayerView.shareButton.behavior.baseURL = URL(string: shareBaseUrl)
        }

        if let ctaWidth = config.ctaWidth {
            resultConfig.ctaButton.width = ctaWidth.width()
            resultConfig.fullScreenPlayerView.ctaButton.width = ctaWidth.width()
        }

        if let ctaDelay = config.ctaDelay,
           let type = ctaDelay.type,
           let value = ctaDelay.value {
            switch type {
            case .constant:
                resultConfig.ctaButton.behavior.delay = .constant(value)
                resultConfig.fullScreenPlayerView.ctaButton.behavior.delay = .constant(value)
            case .percentage:
                resultConfig.ctaButton.behavior.delay = .percentage(Float(value))
                resultConfig.fullScreenPlayerView.ctaButton.behavior.delay = .percentage(Float(value))
            }
        }

        if let ctaHighlightDelay = config.ctaHighlightDelay,
           let type = ctaHighlightDelay.type,
           let value = ctaHighlightDelay.value {
            switch type {
            case .constant:
                resultConfig.ctaButton.behavior.highlightDelay = .constant(value)
                resultConfig.fullScreenPlayerView.ctaButton.behavior.highlightDelay = .constant(value)
            case .percentage:
                resultConfig.ctaButton.behavior.highlightDelay = .percentage(Float(value))
                resultConfig.fullScreenPlayerView.ctaButton.behavior.highlightDelay = .percentage(Float(value))
            }
        }

        if let buttonConfiguration = config.buttonConfiguration {
            if let videoDetailButtonDisplay = buttonConfiguration.videoDetailButton?.getButtonDisplayConfiguration() {
                resultConfig.fullScreenPlayerView.videoDetailButtonDisplayConfiguration = videoDetailButtonDisplay
            }

            if let closeButtonDisplay = buttonConfiguration.closeButton?.getButtonDisplayConfiguration() {
                resultConfig.fullScreenPlayerView.closeButtonConfiguration.closeDisplay = closeButtonDisplay
            }

            if let muteButtonDisplay = buttonConfiguration.muteButton?.getButtonDisplayConfiguration() {
                resultConfig.fullScreenPlayerView.muteButton.muteDisplay = muteButtonDisplay
            }

            if let unmuteButtonDisplay = buttonConfiguration.unmuteButton?.getButtonDisplayConfiguration() {
                resultConfig.fullScreenPlayerView.muteButton.unmuteDisplay = unmuteButtonDisplay
            }

            if let playButtonDisplay = buttonConfiguration.playButton?.getButtonDisplayConfiguration() {
                resultConfig.fullScreenPlayerView.playbackButton.playDisplay = playButtonDisplay
            }

            if let pauseButtonDisplay = buttonConfiguration.pauseButton?.getButtonDisplayConfiguration() {
                resultConfig.fullScreenPlayerView.playbackButton.pauseDisplay = pauseButtonDisplay
            }
        }

        if let adBadge = FireworkSDKPlugin.convertToFWAdBadgeConfiguration(FireworkSDKPlugin.adBadgeConfiguration) {
            resultConfig.adBadge = adBadge
        }

        if let videoPlayerLogoConfiguration = config.videoPlayerLogoConfiguration {
            switch videoPlayerLogoConfiguration.option {
            case .disabled:
                resultConfig.logoConfiguration.option = .disabled
                resultConfig.fullScreenPlayerView.logoConfiguration.option = .disabled
            case .creator:
                if let encodedId = videoPlayerLogoConfiguration.encodedId {
                    resultConfig.logoConfiguration.option = .creator(encodedId)
                    resultConfig.fullScreenPlayerView.logoConfiguration.option = .creator(encodedId)
                }
            case .channelAggregator:
                if let encodedId = videoPlayerLogoConfiguration.encodedId {
                    resultConfig.logoConfiguration.option = .channelAggregator(encodedId)
                    resultConfig.fullScreenPlayerView.logoConfiguration.option = .channelAggregator(encodedId)
                }
            }
        }

        storyBlockVC.viewConfiguration = resultConfig
    }
}
// swiftlint:enable type_body_length
