//
//  ShoppingPlugin.swift
//  fw_flutter_sdk
//
//  Created by Jeff Zheng on 2022/2/21.
//
import FireworkVideo
import Flutter
import Foundation

// swiftlint:disable file_length
class ShoppingPlugin: NSObject, FlutterPlugin, FireworkVideoShoppingDelegate {
    private static var hasRegistered = false
    static let shared = ShoppingPlugin()
    
    var channel: FlutterMethodChannel? = nil

    private var productInfoViewConfigurator: ProductInfoViewConfigurable?

    private var cartIconVisible: Bool?
    private var itemCounts = 0
    private var productInfoViewConfiguration: ProductInfoViewConfiguration?
    private var customClickLinkButtonEnabled = false
    private var customTapProductCardEnabled = false
    private var ctaCompletionHandlerMap: [String: ShoppingCTAHandler] = [:]

    static func register(with registrar: FlutterPluginRegistrar) {
        let channel = FlutterMethodChannel(
            name: ChannelName.shoppingModule.rawValue,
            binaryMessenger: registrar.messenger())
        let instance = hasRegistered ? ShoppingPlugin() : shared
        instance.channel = channel
        registrar.addMethodCallDelegate(instance, channel: channel)
        hasRegistered = true
    }

    // swiftlint:disable:next cyclomatic_complexity
    func handle(_ call: FlutterMethodCall, result: @escaping FlutterResult) {
        switch call.method {
        case ShoppingPluginMethodName.`init`.rawValue:
            initialize()
            result(nil)
        case ShoppingPluginMethodName.setCartIconVisible.rawValue:
            if let visible = call.arguments as? Bool {
                setCartIconVisible(visible)
            }
            result(nil)
        case ShoppingPluginMethodName.setCartItemCount.rawValue:
            if let count = call.arguments as? Int {
                setCartItemCount(count)
            }
            result(nil)
        case ShoppingPluginMethodName.setProductInfoViewConfiguration.rawValue:
            setProductInfoViewConfiguration(call.arguments as? [String: Any])
            result(nil)
        case ShoppingPluginMethodName.setCustomClickLinkButtonEnabled.rawValue:
            if let enabled = call.arguments as? Bool {
                setCustomClickLinkButtonEnabled(enabled)
            }
            result(nil)
        case ShoppingPluginMethodName.setCustomTapProductCardEnabled.rawValue:
            if let enabled = call.arguments as? Bool {
                setCustomTapProductCardEnabled(enabled)
            }
            result(nil)
        case ShoppingPluginMethodName.showShoppingCTALoader.rawValue:
            result(nil)
        case ShoppingPluginMethodName.completeShoppingCTAHandlingProcess.rawValue:
            handleShoppingCTAResult(call.arguments)
            result(nil)
        default:
            result(FlutterMethodNotImplemented)
        }
    }

    func initialize() {
        FireworkVideoSDK.shopping.shoppingDelegate = self
        FireworkVideoSDK.shopping.cartAction = .custom
    }

    func setCartIconVisible(_ visible: Bool) {
        cartIconVisible = visible

        guard let rProductInfoViewConfigurator = productInfoViewConfigurator else {
            return
        }

        rProductInfoViewConfigurator.shoppingCartIconConfiguration.isHidden = !visible
    }

    func setCartItemCount(_ itemCounts: Int) {
        self.itemCounts = itemCounts

        guard let rProductInfoViewConfigurator = productInfoViewConfigurator else {
            return
        }

        rProductInfoViewConfigurator.shoppingCartIconConfiguration.indicator.isHidden = itemCounts == 0
    }

    func setProductInfoViewConfiguration(_ config: [String: Any]?) {
        let config = Convert.buildProductInfoViewConfiguration(config)
        guard let rConfig = config else {
            return
        }

        productInfoViewConfiguration = rConfig

        guard let rProductInfoViewConfigurator = productInfoViewConfigurator else {
            return
        }

        Self.hydrateProductViewConfig(rConfig, rProductInfoViewConfigurator)
    }

    func setCustomClickLinkButtonEnabled(_ enabled: Bool) {
        customClickLinkButtonEnabled = enabled
    }

    func setCustomTapProductCardEnabled(_ enabled: Bool) {
        customTapProductCardEnabled = enabled
    }

    // MARK: - FireworkVideoShoppingDelegate
    func fireworkShopping(
        _ fireworkShopping: FireworkVideoShopping,
        willDisplayProductInfo productInfoViewConfigurator: ProductInfoViewConfigurable,
        forVideo video: VideoDetails
    ) {
        self.productInfoViewConfigurator = productInfoViewConfigurator
        if let cartIconVisible = self.cartIconVisible {
            productInfoViewConfigurator.shoppingCartIconConfiguration.isHidden = !cartIconVisible
        }
        productInfoViewConfigurator.shoppingCartIconConfiguration.indicator.isHidden = itemCounts == 0
        if let productInfoViewConfiguration = productInfoViewConfiguration {
            Self.hydrateProductViewConfig(productInfoViewConfiguration, productInfoViewConfigurator)
        }
    }

    func fireworkShopping(
        _ fireworkShopping: FireworkVideoShopping, updateDetailsForProducts products: [ProductID],
        forVideo video: VideoDetails, _ productHydrator: ProductHydrating
    ) {
        channel?.invokeMethod(
            ShoppingEventName.updateProductDetails.rawValue, arguments: ["productIds": products]
        ) { (result: Any?) -> Void in
            let products = Convert.buildProducts(result as? [AnyObject])
            guard let rProducts = products else {
                return
            }

            for product in rProducts {
                productHydrator.hydrateProduct(
                    product.productId, { build in
                        return Self.hydrateProduct(product, build)
                    })
            }
        }
    }

    func handleShoppingCTAResult(_ json: Any?) {
        guard let arguments = json as? [String: Any] else {
            return
        }

        guard let callbackId = arguments["callbackId"] as? String else {
            return
        }

        guard let ctaCompletionHandler = ctaCompletionHandlerMap[callbackId] else {
            return
        }

        ctaCompletionHandlerMap.removeValue(forKey: callbackId)

        guard let resultJson = arguments["result"] as? [String: Any] else {
            return
        }

        guard let ctaResult = Convert.buildShoppingCTAResult(resultJson) else {
            return
        }

        if let res = ctaResult.res,
           let tips = ctaResult.tips,
           tips.count > 0 {
            switch res {
            case .success:
                ctaCompletionHandler(.feedbackOnly(.success(message: tips)))
            case .fail:
                ctaCompletionHandler(.feedbackOnly(.failure(message: tips)))
            }
        } else {
            ctaCompletionHandler(.none)
        }
    }

    func fireworkShopping(_ fireworkShopping: FireworkVideoShopping,
                          productVariantCTASelected item: SelectedProductVariant,
                          fromVideo video: VideoDetails,
                          ctaCompletionHandler: @escaping ShoppingCTAHandler
    ) {
        let callbackId = UUID().uuidString
        ctaCompletionHandlerMap[callbackId] = ctaCompletionHandler
        var arguments = item.jsObject
        arguments["callbackId"] = callbackId
        channel?.invokeMethod(
            ShoppingEventName.ctaButtonClick.rawValue,
            arguments: arguments
        )
    }

    func fireworkShopping(
        _ fireworkShopping: FireworkVideoShopping,
        didTapLinkButtonAt item: SelectedProductVariant,
        withURL itemURL: String
    ) -> Bool {
        if customClickLinkButtonEnabled {
            channel?.invokeMethod(ShoppingEventName.customLinkButtonClick.rawValue, arguments: item.jsObject)
        }

        return customClickLinkButtonEnabled
    }

    func fireworkShopping(
        _ fireworkShopping: FireworkVideoShopping,
        didTapCartIconForVideo videoDetails: VideoDetails
    ) {
        channel?.invokeMethod(ShoppingEventName.clickCartIcon.rawValue, arguments: [:])
    }

    func fireworkShopping(
        _ fireworkShopping: FireworkVideoShopping,
        didTapProductVariant item: SelectedProductVariant,
        forVideo video: VideoDetails
    ) -> Bool {
        if customTapProductCardEnabled {
            var arguments = item.jsObject
            arguments["video"] = video.jsObject
            channel?.invokeMethod(ShoppingEventName.customProductCardTap.rawValue, arguments: arguments)

        }
        return customTapProductCardEnabled
    }
}

extension ShoppingPlugin {
    // swiftlint:disable cyclomatic_complexity
    // swiftlint:disable function_body_length
    fileprivate static func hydrateProductViewConfig(
        _ config: ProductInfoViewConfiguration,
        _ productInfoViewConfigurator: ProductInfoViewConfigurable
    ) {
        var productDetailsConfiguration = productInfoViewConfigurator.productDetailsConfiguration

        if let ctaButtonConfiguration = config.ctaButton {
            if let text = ctaButtonConfiguration.text {
                switch text {
                case .addToCart:
                    productDetailsConfiguration.ctaButton.text = .addToCart
                case .shopNow:
                    productDetailsConfiguration.ctaButton.text = .shopNow
                }
            }
            if let backgroundColor = ctaButtonConfiguration.backgroundColor,
               backgroundColor.count > 0,
               let color = backgroundColor.uicolor() {
                productDetailsConfiguration.ctaButton.buttonConfiguration.backgroundColor = color
            }

            if let textColor = ctaButtonConfiguration.textColor,
               textColor.count > 0,
               let color = textColor.uicolor() {
                productDetailsConfiguration.ctaButton.buttonConfiguration.textColor = color
            }

            if let fontSize = ctaButtonConfiguration.fontSize {
                let iOSFontInfo = ctaButtonConfiguration.iOSFontInfo ?? FontInfo()
                productDetailsConfiguration.ctaButton.buttonConfiguration.font = iOSFontInfo.getFont(fontSize)
            }
        }

        if let linkButtonConfiguration = config.linkButton {
            if let isHidden = linkButtonConfiguration.isHidden {
                productDetailsConfiguration.linkButtonConfiguration.isHidden = isHidden
            }
        }

        if let productCardConfiguration = config.productCard {
            if let theme = productCardConfiguration.theme {
                switch theme {
                case .light:
                    productInfoViewConfigurator.productCardConfiguration.appearance = .light
                case .dark:
                    productInfoViewConfigurator.productCardConfiguration.appearance = .dark
                }
            }

            if let text = productCardConfiguration.ctaButtonText {
                switch text {
                case .shopNow:
                    productInfoViewConfigurator.productCardConfiguration.ctaConfig.text = .shopNow
                case .buyNow:
                    productInfoViewConfigurator.productCardConfiguration.ctaConfig.text = .buyNow
                }
            }

            if let cornerRadius = productCardConfiguration.cornerRadius {
                productInfoViewConfigurator.productCardConfiguration.cornerRadius = cornerRadius
            }

            if let priceConfiguration = productCardConfiguration.priceConfiguration {
//                if let isPriceHidden = priceConfiguration.isHidden {
//                    productInfoViewConfigurator.productCardConfiguration.priceConfiguration.isHidden = isPriceHidden
//                }
                if let axis = priceConfiguration.axis {
                    switch axis {
                    case .horizontal:
                        productInfoViewConfigurator.productCardConfiguration.priceConfiguration.axis = .horizontal
                    case .vertical:
                        productInfoViewConfigurator.productCardConfiguration.priceConfiguration.axis = .vertical
                    }
                }

                if let priceLabel = priceConfiguration.priceLabel {
                    if let textColor = priceLabel.textColor,
                       textColor.count > 0,
                        let color = textColor.uicolor() {
                    productInfoViewConfigurator.productCardConfiguration.priceConfiguration.priceLabel.textColor = color
                    }
                    if let fontSize = priceLabel.fontSize {
                        let iOSFontInfo = priceLabel.iOSFontInfo ?? FontInfo()
                        productInfoViewConfigurator
                            .productCardConfiguration
                            .priceConfiguration
                            .priceLabel
                            .font = iOSFontInfo.getFont(fontSize)
                    }
                    if let numberOfLines = priceLabel.numberOfLines {
                        productInfoViewConfigurator
                            .productCardConfiguration
                            .priceConfiguration
                            .priceLabel
                            .numberOfLines = numberOfLines
                    }
                }

                if let originalPriceLabel = priceConfiguration.originalPriceLabel {
                    if let textColor = originalPriceLabel.textColor,
                       textColor.count > 0,
                        let color = textColor.uicolor() {
                        productInfoViewConfigurator
                            .productCardConfiguration
                            .priceConfiguration
                            .originalPriceLabel
                            .textColor = color
                    }
                    if let fontSize = originalPriceLabel.fontSize {
                        let iOSFontInfo = originalPriceLabel.iOSFontInfo ?? FontInfo()
                        productInfoViewConfigurator
                            .productCardConfiguration
                            .priceConfiguration
                            .originalPriceLabel
                            .font = iOSFontInfo.getFont(fontSize)
                    }
                    if let numberOfLines = originalPriceLabel.numberOfLines {
                        productInfoViewConfigurator
                            .productCardConfiguration
                            .priceConfiguration
                            .originalPriceLabel
                            .numberOfLines = numberOfLines
                    }
                }

                if let isPriceFirst = priceConfiguration.isPriceFirst {
                    productInfoViewConfigurator.productCardConfiguration.priceConfiguration.isPriceFirst = isPriceFirst
                }
            }

            if let isCtaButtonHidden = productCardConfiguration.isCtaButtonHidden {
                productInfoViewConfigurator.productCardConfiguration.ctaConfig.isHidden = isCtaButtonHidden
            }

            if let ctaButtonStyle = productCardConfiguration.ctaButtonStyle {
                if let textColor = ctaButtonStyle.textColor,
                   textColor.count > 0,
                   let color = textColor.uicolor() {
                    productInfoViewConfigurator.productCardConfiguration.ctaConfig.textColor = color
                }

                if let fontSize = ctaButtonStyle.fontSize {
                    let iOSFontInfo = ctaButtonStyle.iOSFontInfo ?? FontInfo()
                    productInfoViewConfigurator.productCardConfiguration.ctaConfig.font = iOSFontInfo.getFont(fontSize)
                }
            }

            if let width = productCardConfiguration.width, let height = productCardConfiguration.height {
                productInfoViewConfigurator.productCardConfiguration.size = CGSize(
                    width: width,
                    height: height
                )
            } else if let width = productCardConfiguration.width {
                let originalSize = productInfoViewConfigurator.productCardConfiguration.size
                productInfoViewConfigurator.productCardConfiguration.size = CGSize(
                    width: width,
                    height: originalSize.height
                )
            } else if let height = productCardConfiguration.height {
                let originalSize = productInfoViewConfigurator.productCardConfiguration.size
                productInfoViewConfigurator.productCardConfiguration.size = CGSize(
                    width: originalSize.width,
                    height: height
                )
            }

            if let backgroundColor = productCardConfiguration.backgroundColor,
               backgroundColor.count > 0,
               let color = backgroundColor.uicolor() {
               productInfoViewConfigurator.productCardConfiguration.backgroundColor = color
            }

            if let iconConfiguration = productCardConfiguration.iconConfiguration {
                if let cornerRadius = iconConfiguration.cornerRadius {
                    productInfoViewConfigurator.productCardConfiguration.productIcon.cornerRadius = cornerRadius
                }
            }

            if let nameLabel = productCardConfiguration.nameLabel {
                if let textColor = nameLabel.textColor,
                   textColor.count > 0,
                    let color = textColor.uicolor() {
                    productInfoViewConfigurator.productCardConfiguration.nameLabel.textColor = color
                }
                if let fontSize = nameLabel.fontSize {
                    let iOSFontInfo = nameLabel.iOSFontInfo ?? FontInfo()
                    productInfoViewConfigurator.productCardConfiguration.nameLabel.font = iOSFontInfo.getFont(fontSize)
                }
                if let numberOfLines = nameLabel.numberOfLines {
                    productInfoViewConfigurator.productCardConfiguration.nameLabel.numberOfLines = numberOfLines
                }
            }

        }

        productInfoViewConfigurator.productDetailsConfiguration = productDetailsConfiguration
    }
    // swiftlint:enable cyclomatic_complexity
    // swiftlint:enable function_body_length
}

extension ShoppingPlugin {
    fileprivate static func hydrateProduct(_ product: Product, _ pbuild: ProductBuilder)
        -> ProductBuilder {
            if let name = product.name {
                pbuild.name(name)
            }
            if let description = product.description {
                pbuild.description(description)
            }
            if let isAvailable = product.isAvailable {
                pbuild.isAvailable(isAvailable)
            }
            if let variants = product.units {
                for variant in variants {
                    pbuild.variant(variant.unitId) { pvBuild in
                        return Self.buildVariant(variant, pvBuild)
                    }
                }
            }
        return pbuild
    }

    fileprivate static func buildVariant(
        _ variant: Product.ProductUnit, _ pvBuild: ProductVariantBuilder
    ) -> ProductVariantBuilder {
        if let name = variant.name {
            pvBuild.name(name)
        }
        if let url = variant.url {
            pvBuild.url(url)
        }
        if let imageUrl = variant.imageUrl {
            pvBuild.imageUrl(imageUrl)
        }
        if let price = variant.price {
            if let amount = price.amount, let currencyCode = price.currencyCode {
                pvBuild.formattedPrice(Decimal(amount), currencyCode: currencyCode)
            }
        }
        if let isAvailable = variant.isAvailable {
            pvBuild.isAvailable(isAvailable)
        }
        let options = Self.buildVariantOption(variant.options)
        if let options = options {
            pvBuild.options(options)
        }
        return pvBuild
    }

    fileprivate static func buildVariantOption(_ options: [Product.VariantOption]?) -> [String: String]? {
        guard let options = options else {
            return nil
        }
        var result: [String: String] = [:]
        for option in options {
            guard let name = option.name, let value = option.value else {
                continue
            }
            result[name] = value
        }
        return result
    }
}
// swiftlint:enable file_length
