import 'dart:async';

import 'package:flutter/foundation.dart';
import 'package:flutter/material.dart';
import 'package:fw_flutter_sdk/src/utils/fw_json_util.dart';
import 'package:fw_flutter_sdk/src/utils/model_utils/product_info_view_configuration_util.dart';

import '../../constants/fw_event_name.dart';
import '../../firework_sdk.dart';
import '../../fw_event_bus.dart';
import '../../models/ad_configuration.dart';
import '../../models/button_info.dart';
import '../../models/fw_error.dart';
import '../../models/story_block_configuration.dart';
import '../../models/story_block_native_configuration.dart';
import '../../models/story_block_source.dart';
import '../../utils/fw_global_state.dart';
import 'story_block_controller.dart';
import 'story_block_platform_handler.dart';
import 'story_block_platform_view.dart';

typedef StoryBlockLoadFinishedCallback = void Function(FWError? error);
typedef StoryBlockEmptyCallback = void Function(FWError? error);
typedef StoryBlockCreatedCallback = void Function(
    StoryBlockController controller);
typedef StoryBlockDidStartPictureInPicture = void Function(FWError? error);
typedef StoryBlockDidStopPictureInPicture = void Function(FWError? error);

class StoryBlock extends StatefulWidget {
  /// The source of the story block.
  final StoryBlockSource source;

  /// The channel id of the story block.
  /// Required when the source is set as `channel` or `playlist` or `dynamicContent`.
  final String? channel;

  /// The playlist id of the story block.
  /// Required when the source is set as `playlist`.
  final String? playlist;

  /// The dynamic content parameters of the story block.
  /// Required when the source is set as `dynamicContent`.
  final Map<String, List<String>>? dynamicContentParameters;

  /// Hashtag filter expression is an s-expression used to provide feeds filtered by hashtags with specified criteria.
  /// Queries are specified with boolean predicates on what hashtags are there on the video.
  /// For instance, (and sport food) (or sport food) (and sport (or food comedy)) sport are all valid expressions.
  /// Non-UTF-8 characters are not allowed. If using boolean predicates, the expression needs to be wrapped with parenthesis.
  /// Required when the source is set as hashtagPlaylist.
  final String? hashtagFilterExpression;

  /// Product ids used to generate the sku feed.
  /// Required when the source is set as sku.
  final List<String>? productIds;

  /// The video or live stream id.
  final String? contentId;

  /// Specifies if Picture in Picture is enabled.
  final bool? enablePictureInPicture;

  /// The corner radius of story block.
  final double? cornerRadius;

  /// Ad configuration of the feed.
  final AdConfiguration? adConfiguration;

  /// The configuration of the story block.
  final StoryBlockConfiguration? storyBlockConfiguration;

  /// The width of the story block.
  final double? width;

  /// The height of the story block.
  final double? height;

  /// Set this property to true if you want keep the widget alive during scrolling list out of view box.
  /// Default to false.
  final bool? wantKeepAlive;

  /// The callback of story block creation.
  final StoryBlockCreatedCallback? onStoryBlockCreated;

  /// The callback of the feed loading result.
  /// It means loading successfully when error is equal to null.
  final StoryBlockLoadFinishedCallback? onStoryBlockLoadFinished;

  /// The callback is triggered when there are no items in the story block.
  /// The callback is triggered in the following cases:
  ///  1. Loading successfully but the back end returns an empty list.
  ///  2. The load failed and list is empty.
  /// onStoryBlockLoadFinished will also be triggered when onStoryBlockEmpty is triggered.
  /// Only supported on Android.
  final StoryBlockEmptyCallback? onStoryBlockEmpty;

  /// The callback for platform view did start picture in picture
  /// Only supported on iOS
  final StoryBlockDidStartPictureInPicture?
      onStoryBlockDidStartPictureInPicture;

  /// The callback for platform view did stop picture in picture
  /// Only supported on iOS
  final StoryBlockDidStopPictureInPicture? onStoryBlockDidStopPictureInPicture;

  const StoryBlock({
    Key? key,
    required this.source,
    this.channel,
    this.playlist,
    this.dynamicContentParameters,
    this.hashtagFilterExpression,
    this.productIds,
    this.contentId,
    this.enablePictureInPicture,
    this.cornerRadius,
    this.adConfiguration,
    this.storyBlockConfiguration,
    this.width,
    this.height,
    this.wantKeepAlive,
    this.onStoryBlockCreated,
    this.onStoryBlockLoadFinished,
    this.onStoryBlockEmpty,
    this.onStoryBlockDidStartPictureInPicture,
    this.onStoryBlockDidStopPictureInPicture,
  }) : super(key: key);

  @override
  State<StoryBlock> createState() => _StoryBlockState();
}

class _StoryBlockState extends State<StoryBlock>
    with AutomaticKeepAliveClientMixin {
  StoryBlockController? _controller;
  StreamSubscription? _eventSubscription;
  bool _sdkInitCalled = false;

  @override
  void initState() {
    super.initState();
    _eventSubscription = FWEventBus.getInstance().on().listen((event) {
      if (mounted) {
        if (event.eventName == FWEventName.adBadgeConfigurationUpdated) {
          setState(() {});
        } else if (event.eventName == FWEventName.shareBaseURLUpdated) {
          setState(() {});
        } else if (event.eventName == FWEventName.videoLaunchBehaviorUpdated) {
          setState(() {});
        } else if (event.eventName == FWEventName.appLanguageUpdated) {
          setState(() {});
        } else if (event.eventName ==
            FWEventName.productInfoViewConfigurationUpdated) {
          setState(() {});
        }
      }
    });

    _sdkInitCalled = FWGlobalState.getInstance().sdkInitCalled;
    if (!_sdkInitCalled) {
      WidgetsBinding.instance.addPostFrameCallback((_) {
        FWGlobalState.getInstance().sdkInitCalledFuture.then((__) {
          setState(() {
            _sdkInitCalled = true;
          });
        });
      });
    }
  }

  @override
  void dispose() {
    _eventSubscription?.cancel();
    super.dispose();
  }

  @override
  void didUpdateWidget(covariant StoryBlock oldWidget) {
    super.didUpdateWidget(oldWidget);
    if (oldWidget.wantKeepAlive != widget.wantKeepAlive) {
      updateKeepAlive();
    }
  }

  @override
  Widget build(BuildContext context) {
    super.build(context);

    if (!_sdkInitCalled) {
      return SizedBox(
        width: widget.width,
        height: widget.height,
      );
    }

    return SizedBox(
      width: widget.width,
      height: widget.height,
      child: StoryBlockPlatformView(
        key: ValueKey(_generateKey()),
        source: widget.source,
        channel: widget.channel,
        playlist: widget.playlist,
        dynamicContentParameters: widget.dynamicContentParameters,
        hashtagFilterExpression: widget.hashtagFilterExpression,
        productIds: widget.productIds,
        contentId: widget.contentId,
        cornerRadius: widget.cornerRadius,
        adConfiguration: widget.adConfiguration,
        storyBlockConfiguration:
            StoryBlockNativeConfiguration.withStoryBlockConfiguration(
          storyBlockConfiguration: widget.storyBlockConfiguration?.deepCopy(),
          enablePictureInPicture: widget.enablePictureInPicture,
        ),
        onStoryBlockformViewCreated: _onStoryBlockPlatformViewCreated,
        onStoryBlockPlatformViewLoadFinished: widget.onStoryBlockLoadFinished,
        onStoryBlockPlatformViewEmpty: widget.onStoryBlockEmpty,
        onStoryBlockPlatformViewDidStartPictureInPicture:
            widget.onStoryBlockDidStartPictureInPicture,
        onStoryBlockPlatformViewDidStopPictureInPicture:
            widget.onStoryBlockDidStopPictureInPicture,
      ),
    );
  }

  String _generateKey() {
    final gShareBaseURL = FireworkSDK.getInstance().shareBaseURL;
    final appLanguage = FireworkSDK.getInstance().appLanguage;
    final videoLaunchBehavior =
        FireworkSDK.getInstance().videoLaunchBehavior?.name;
    final productInfoViewConfiguration =
        FireworkSDK.getInstance().shopping.productInfoViewConfiguration;
    final adBadgeConfiguration = FireworkSDK.getInstance().adBadgeConfiguration;
    final adBadgeTextType = adBadgeConfiguration?.badgeTextType?.name;
    final backgroundColorOfAdBadge = adBadgeConfiguration?.backgroundColor;
    final textColorOfAdBadge = adBadgeConfiguration?.textColor;
    final androidFontIsCustomOfAdBadge =
        adBadgeConfiguration?.androidFontInfo?.isCustom;
    final androidFontTypefaceNameOfAdBadge =
        adBadgeConfiguration?.androidFontInfo?.typefaceName;

    final source = widget.source.name;
    final channel = widget.channel;
    final playlist = widget.playlist;
    final dynamicContentParametersString =
        _generateDynamicContentParametersString();
    final hashtagFilterExpression = widget.hashtagFilterExpression;
    final productIds = widget.productIds?.join(",");
    final contentId = widget.contentId;
    final enablePictureInPicture = widget.enablePictureInPicture;
    final cornerRadius = widget.cornerRadius?.toStringAsFixed(5);

    final playerStyle = widget.storyBlockConfiguration?.playerStyle?.name;
    final videoCompleteAction =
        widget.storyBlockConfiguration?.videoCompleteAction?.name;
    final showShareButton = widget.storyBlockConfiguration?.showShareButton;
    final showPlaybackButton =
        widget.storyBlockConfiguration?.showPlaybackButton;
    final showMuteButton = widget.storyBlockConfiguration?.showMuteButton;
    final showBranding = widget.storyBlockConfiguration?.showBranding;
    final ctaDelay = widget.storyBlockConfiguration?.ctaDelay;
    final ctaDelayType = ctaDelay?.type.name;
    final ctaHighlightDelay = widget.storyBlockConfiguration?.ctaHighlightDelay;
    final ctaHighlightDelayType = ctaHighlightDelay?.type.name;
    final ctaHighlightDelayValue = ctaHighlightDelay?.value.toStringAsFixed(5);
    final ctaDelayValue = ctaDelay?.value.toStringAsFixed(5);
    final shareBaseURL = widget.storyBlockConfiguration?.shareBaseURL;
    final ctaWidth = widget.storyBlockConfiguration?.ctaWidth?.name;
    final buttonConfiguration =
        widget.storyBlockConfiguration?.buttonConfiguration;
    final showVideoDetailTitle =
        widget.storyBlockConfiguration?.showVideoDetailTitle;

    final requiresAds = widget.adConfiguration?.requiresAds;
    final adsFetchTimeout = widget.adConfiguration?.adsFetchTimeout;
    final vastAttributesString = _generateVastAttributesString();

    String key = "_gShareBaseURL:$gShareBaseURL";
    if (defaultTargetPlatform == TargetPlatform.iOS) {
      key += "_appLanguage:$appLanguage";
    }
    key += "_videoLaunchBehavior:$videoLaunchBehavior";
    if (productInfoViewConfiguration != null) {
      final productInfoViewConfigurationJson =
          ProductInfoViewConfigurationUtil.toJson(productInfoViewConfiguration);
      final productInfoViewConfigurationJsonKey =
          FWJsonUtil.generateJsonKey(productInfoViewConfigurationJson);
      key +=
          "_productInfoViewConfiguration:$productInfoViewConfigurationJsonKey";
    }
    key += "_adBadgeTextType:$adBadgeTextType";
    key += "_backgroundColorOfAdBadge:$backgroundColorOfAdBadge";
    key += "_textColorOfAdBadge:$textColorOfAdBadge";
    if (defaultTargetPlatform == TargetPlatform.android) {
      key += "_androidFontIsCustomOfAdBadge:$androidFontIsCustomOfAdBadge";
      key +=
          "_androidFontTypefaceNameOfAdBadge:$androidFontTypefaceNameOfAdBadge";
    }
    key += "_source:$source";
    key += "_channel:$channel";
    key += "_playlist:$playlist";
    key += "_dynamicContentParameters:$dynamicContentParametersString";
    key += "_hashtagFilterExpression:$hashtagFilterExpression";
    key += "_productIds:$productIds";
    key += "_contentId:$contentId";
    key += "_enablePictureInPicture:$enablePictureInPicture";

    if (defaultTargetPlatform == TargetPlatform.android) {
      key += "_cornerRadius:$cornerRadius";
    }
    key += "_playerStyle:$playerStyle";
    key += "_videoCompleteAction:$videoCompleteAction";
    key += "_showShareButton:$showShareButton";
    key += "_showPlaybackButton:$showPlaybackButton";
    key += "_showMuteButton:$showMuteButton";
    key += "_showBranding:$showBranding";
    key += "_ctaDelayType:$ctaDelayType";
    key += "_ctaDelayValue:$ctaDelayValue";
    key += "_ctaHighlightDelayType:$ctaHighlightDelayType";
    key += "_ctaHighlightDelayValue:$ctaHighlightDelayValue";
    key += "_shareBaseURL:$shareBaseURL";
    key += "_ctaWidth:$ctaWidth";
    key +=
        "_buttonConfiguration.videoDetailButton:${_generateButtonInfoString(buttonConfiguration?.videoDetailButton)}";
    key +=
        "_buttonConfiguration.closeButton:${_generateButtonInfoString(buttonConfiguration?.closeButton)}";
    key +=
        "_buttonConfiguration.muteButton:${_generateButtonInfoString(buttonConfiguration?.muteButton)}";
    key +=
        "_buttonConfiguration.unmuteButton:${_generateButtonInfoString(buttonConfiguration?.unmuteButton)}";
    key +=
        "_buttonConfiguration.playButton:${_generateButtonInfoString(buttonConfiguration?.playButton)}";
    key +=
        "_buttonConfiguration.pauseButton:${_generateButtonInfoString(buttonConfiguration?.pauseButton)}";
    key += "_showVideoDetailTitle:$showVideoDetailTitle";

    key += "_requiresAds:$requiresAds";
    key += "_adsFetchTimeout:$adsFetchTimeout";
    key += "_vastAttributesString:$vastAttributesString";

    return key;
  }

  String _generateDynamicContentParametersString() {
    final parameters = widget.dynamicContentParameters;
    if (parameters == null) {
      return "";
    }

    var resultString = "";
    final sortedKeyList = parameters.keys.toList()..sort();
    for (final key in sortedKeyList) {
      final value = parameters[key];
      if (value != null) {
        final valueString = value.join(",");
        if (resultString.isNotEmpty) {
          resultString += "_";
        }
        resultString += "$key:$valueString";
      }
    }

    return resultString;
  }

  String _generateVastAttributesString() {
    final vastAttributes = widget.adConfiguration?.vastAttributes;
    if (vastAttributes == null) {
      return "";
    }

    var resultString = "";
    for (var attribute in vastAttributes) {
      final name = attribute.name;
      final value = attribute.value;
      if (resultString.isNotEmpty) {
        resultString += "_";
      }
      resultString += "$name:$value";
    }
    return resultString;
  }

  String _generateButtonInfoString(ButtonInfo? buttonInfo) {
    return "imageName:${buttonInfo?.imageName}_systemImageName:${buttonInfo?.systemImageName}_tintColor:${buttonInfo?.tintColor}";
  }

  void _onStoryBlockPlatformViewCreated(StoryBlockPlatformHandler handler) {
    if (_controller == null) {
      _controller = StoryBlockController(handler: handler);
      widget.onStoryBlockCreated?.call(_controller!);
    } else {
      _controller!.handler = handler;
    }
  }

  @override
  bool get wantKeepAlive => widget.wantKeepAlive ?? false;
}
