import 'package:flutter/foundation.dart';
import 'package:flutter/gestures.dart';
import 'package:flutter/material.dart';
import 'package:flutter/rendering.dart';
import 'package:flutter/services.dart';
import 'package:fw_flutter_sdk/src/utils/fw_logger_util.dart';
import 'package:visibility_detector/visibility_detector.dart';

import '../../models/ad_configuration.dart';
import '../../models/fw_error.dart';
import '../../models/story_block_native_configuration.dart';
import '../../models/story_block_source.dart';
import '../../utils/model_utils/ad_configuration_util.dart';
import '../../utils/model_utils/story_block_configuration_util.dart';
import '../mixins/android_platform_view_mixin.dart';
import 'story_block_platform_handler.dart';
import 'story_block_native_callbacks.dart';

typedef StoryBlockPlatformViewCreatedCallback = void Function(
    StoryBlockPlatformHandler handler);

class StoryBlockPlatformView extends StatefulWidget {
  final StoryBlockSource source;
  final String? channel;
  final String? playlist;
  final Map<String, List<String>>? dynamicContentParameters;
  final String? hashtagFilterExpression;
  final List<String>? productIds;
  final String? contentId;
  final double? cornerRadius;
  final AdConfiguration? adConfiguration;
  final StoryBlockNativeConfiguration? storyBlockConfiguration;
  final StoryBlockPlatformViewCreatedCallback? onStoryBlockformViewCreated;
  final StoryBlockPlatformViewLoadFinishedCallback?
      onStoryBlockPlatformViewLoadFinished;
  final StoryBlockPlatformViewEmptyCallback? onStoryBlockPlatformViewEmpty;
  final StoryBlockPlatformViewDidStartPictureInPicture?
      onStoryBlockPlatformViewDidStartPictureInPicture;
  final StoryBlockPlatformViewDidStopPictureInPicture?
      onStoryBlockPlatformViewDidStopPictureInPicture;

  const StoryBlockPlatformView({
    Key? key,
    required this.source,
    this.channel,
    this.playlist,
    this.dynamicContentParameters,
    this.hashtagFilterExpression,
    this.productIds,
    this.contentId,
    this.cornerRadius,
    this.adConfiguration,
    this.storyBlockConfiguration,
    this.onStoryBlockformViewCreated,
    this.onStoryBlockPlatformViewLoadFinished,
    this.onStoryBlockPlatformViewEmpty,
    this.onStoryBlockPlatformViewDidStartPictureInPicture,
    this.onStoryBlockPlatformViewDidStopPictureInPicture,
  }) : super(key: key);

  @override
  State<StoryBlockPlatformView> createState() => _StoryBlockPlatformViewState();
}

class _StoryBlockPlatformViewState extends State<StoryBlockPlatformView>
    with AndroidPlatformViewMixin {
  static int visibilityDetectorKeyIdFactor = 0;
  final String _viewType = "fw.fondor.view/story_block";
  StoryBlockPlatformHandler? _handler;
  final _iOSPlatformViewKey = GlobalKey();
  RenderUiKitView? _renderUiKitView;
  Color? _androidPlatformViewBackgroundColor = Colors.black;
  bool _loaded = false;
  int _visibilityDetectorKeyId = 0;
  bool? _visible;

  @override
  void initState() {
    super.initState();
    _visibilityDetectorKeyId = visibilityDetectorKeyIdFactor;
    visibilityDetectorKeyIdFactor++;
  }

  @override
  void didUpdateWidget(covariant StoryBlockPlatformView oldWidget) {
    super.didUpdateWidget(oldWidget);
    if (oldWidget.storyBlockConfiguration != widget.storyBlockConfiguration) {
      _handler?.updateStoryBlockConfiguration(widget.storyBlockConfiguration);
    }
  }

  @override
  void dispose() {
    super.dispose();
    FWLoggerUtil.log("_StoryBlockPlatformViewState dispose");
    _handler?.removeIvsPlayerView();
  }

  @override
  Widget build(BuildContext context) {
    final cornerRadius = widget.cornerRadius;

    if (defaultTargetPlatform == TargetPlatform.android) {
      return VisibilityDetector(
        key:
            Key("firework-story-block-platform-view-$_visibilityDetectorKeyId"),
        child: Container(
          decoration: BoxDecoration(
            borderRadius: BorderRadius.circular(cornerRadius ?? 0),
            color: _androidPlatformViewBackgroundColor,
          ),
          child: buildAndroidPlatformView(_buildAndroidPlatformView),
        ),
        onVisibilityChanged: (visibilityInfo) {
          FWLoggerUtil.log(
              "_StoryBlockPlatformViewState onVisibilityChanged ${visibilityInfo.visibleFraction}");
          if (visibilityInfo.visibleFraction > 0) {
            if (_visible == null || _visible == false) {
              _handler?.onViewportEntered();
              _handler?.showIvsPlayerView();
            }
            _visible = true;
          } else {
            if (_visible == null || _visible == true) {
              _handler?.onViewportLeft();
              _handler?.hideIvsPlayerView();
            }
            _visible = false;
          }
        },
      );
    } else {
      return Container(
        decoration: BoxDecoration(
          borderRadius: BorderRadius.circular(cornerRadius ?? 0),
        ),
        clipBehavior: Clip.hardEdge,
        child: _buildIOSPlatformView(context),
      );
    }
  }

  Widget _buildAndroidPlatformView(BuildContext context, ValueKey key) {
    final creationParams = _generateCreationParams();
    const layoutDirection = TextDirection.ltr;
    const creationParamsCodec = StandardMessageCodec();
    return PlatformViewLink(
      key: ValueKey(key),
      viewType: _viewType,
      surfaceFactory:
          (BuildContext context, PlatformViewController controller) {
        return AndroidViewSurface(
          controller: controller as AndroidViewController,
          gestureRecognizers: _defaultGestureRecognizersAndroid(),
          hitTestBehavior: PlatformViewHitTestBehavior.opaque,
        );
      },
      onCreatePlatformView: (PlatformViewCreationParams params) {
        return PlatformViewsService.initExpensiveAndroidView(
          id: params.id,
          viewType: _viewType,
          layoutDirection: layoutDirection,
          creationParams: creationParams,
          creationParamsCodec: creationParamsCodec,
          onFocus: () {
            params.onFocusChanged(true);
          },
        )
          ..addOnPlatformViewCreatedListener(params.onPlatformViewCreated)
          ..addOnPlatformViewCreatedListener(_onPlatformViewCreated)
          ..create();
      },
    );
  }

  Widget _buildIOSPlatformView(BuildContext context) {
    Map<String, dynamic> creationParams = _generateCreationParams();
    const layoutDirection = TextDirection.ltr;

    return UiKitView(
      key: _iOSPlatformViewKey,
      viewType: _viewType,
      onPlatformViewCreated: _onPlatformViewCreated,
      layoutDirection: layoutDirection,
      creationParams: creationParams,
      creationParamsCodec: const StandardMessageCodec(),
      gestureRecognizers: _gestureRecognizersForHidingPdpIOS(),
    );
  }

  Map<String, dynamic> _generateCreationParams() {
    Map<String, dynamic> creationParams = <String, dynamic>{
      "source": widget.source.name,
    };

    if (widget.channel != null) {
      creationParams["channel"] = widget.channel!;
    }

    if (widget.playlist != null) {
      creationParams["playlist"] = widget.playlist!;
    }

    if (widget.dynamicContentParameters != null) {
      creationParams["dynamicContentParameters"] =
          widget.dynamicContentParameters!;
    }

    if (widget.hashtagFilterExpression != null) {
      creationParams["hashtagFilterExpression"] =
          widget.hashtagFilterExpression!;
    }

    if (widget.productIds != null) {
      creationParams["productIds"] = widget.productIds!;
    }

    if (widget.contentId != null) {
      creationParams["contentId"] = widget.contentId!;
    }

    if (defaultTargetPlatform == TargetPlatform.android) {
      if (widget.cornerRadius != null) {
        creationParams["cornerRadius"] = widget.cornerRadius!;
      }
    }

    if (widget.adConfiguration != null) {
      creationParams["adConfiguration"] =
          AdConfigurationUtil.toJson(widget.adConfiguration!);
    }

    if (widget.storyBlockConfiguration != null) {
      creationParams["storyBlockConfiguration"] =
          StoryBlockConfigurationUtil.toJson(widget.storyBlockConfiguration);
    }

    if (defaultTargetPlatform == TargetPlatform.android) {
      creationParams["marginRight"] = androidMarginRight;
    }

    return creationParams;
  }

  Set<Factory<EagerGestureRecognizer>> _gestureRecognizersForShowingPdpIOS() {
    return <Factory<EagerGestureRecognizer>>{
      Factory<EagerGestureRecognizer>(
        () => EagerGestureRecognizer(),
      ),
    };
  }

  Set<Factory<HorizontalDragGestureRecognizer>>
      _gestureRecognizersForHidingPdpIOS() {
    return <Factory<HorizontalDragGestureRecognizer>>{
      Factory<HorizontalDragGestureRecognizer>(
        () => HorizontalDragGestureRecognizer(),
      ),
    };
  }

  Set<Factory<HorizontalDragGestureRecognizer>>
      _defaultGestureRecognizersAndroid() {
    return <Factory<HorizontalDragGestureRecognizer>>{
      Factory<HorizontalDragGestureRecognizer>(
        () => HorizontalDragGestureRecognizer(),
      ),
    };
  }

  void _onPlatformViewCreated(int viewId) {
    final handler = StoryBlockPlatformHandler(
      viewId: viewId,
      callbacks: StoryBlockPlatformHandlerCallbacks(
        onPlatformViewLoadFinished: _onPlatformViewLoadFinished,
        onPlatformViewEmpty: _onPlatformViewEmpty,
        onPlatformViewPdpVisibilityChange: _onPlatformViewPdpVisibilityChange,
        onPlatformViewDidStartPictureInPicture:
            _onPlatformViewDidStartPictureInPicture,
        onPlatformViewDidStopPictureInPicture:
            _onPlatformViewDidStopPictureInPicture,
      ),
    );
    _handler = handler;
    widget.onStoryBlockformViewCreated?.call(handler);

    WidgetsBinding.instance.addPostFrameCallback((_) {
      if (defaultTargetPlatform == TargetPlatform.iOS) {
        if (_iOSPlatformViewKey.currentContext != null) {
          final renderObject =
              _iOSPlatformViewKey.currentContext!.findRenderObject();
          if (renderObject != null) {
            final renderUiKitView = _findRenderUiKitView(renderObject);
            _renderUiKitView = renderUiKitView;
          }
        }
      }
    });
  }

  void _onPlatformViewLoadFinished(FWError? error) {
    widget.onStoryBlockPlatformViewLoadFinished?.call(error);

    if (error == null) {
      setState(() {
        _loaded = true;
        _androidPlatformViewBackgroundColor = Colors.black;
      });
    } else {
      if (_loaded) {
        setState(() {
          _androidPlatformViewBackgroundColor = Colors.black;
        });
      } else {
        widget.onStoryBlockPlatformViewEmpty?.call(error);

        setState(() {
          _androidPlatformViewBackgroundColor = null;
        });
      }
    }
  }

  void _onPlatformViewEmpty(FWError? error) {
    widget.onStoryBlockPlatformViewLoadFinished?.call(error);
    widget.onStoryBlockPlatformViewEmpty?.call(error);

    setState(() {
      _androidPlatformViewBackgroundColor = null;
    });
  }

  void _onPlatformViewPdpVisibilityChange(bool visible) {
    _renderUiKitView?.updateGestureRecognizers(visible
        ? _gestureRecognizersForShowingPdpIOS()
        : _gestureRecognizersForHidingPdpIOS());
  }

  void _onPlatformViewDidStartPictureInPicture(FWError? error) {
    widget.onStoryBlockPlatformViewDidStartPictureInPicture?.call(error);
  }

  void _onPlatformViewDidStopPictureInPicture(FWError? error) {
    widget.onStoryBlockPlatformViewDidStopPictureInPicture?.call(error);
  }

  RenderUiKitView? _findRenderUiKitView(RenderObject object) {
    if (object is RenderUiKitView) {
      return object;
    }
    RenderUiKitView? result;
    object.visitChildren((child) {
      final childResult = _findRenderUiKitView(child);
      if (childResult != null) {
        result = childResult;
      }
    });
    return result;
  }
}
